@@ -1,6 +1,8 @@
-from typing import List, Optional, Tuple, cast
+from typing import List, Optional, Tuple, Union, cast, overload
 from typing_extensions import Final
 
+from .filebytes import FileBytes
+
 
 class BinaryDiffException(Exception):
     pass
@@ -19,7 +21,7 @@ class BinaryDiff:
         return out
 
     @staticmethod
-    def diff(bin1: bytes, bin2: bytes) -> List[str]:
+    def diff(bin1: Union[bytes, FileBytes], bin2: Union[bytes, FileBytes]) -> List[str]:
         binlength = len(bin1)
         if binlength != len(bin2):
             raise BinaryDiffException("Cannot diff different-sized binary blobs!")
@@ -30,8 +32,9 @@ class BinaryDiff:
         # Chunk the differences, assuming files are usually about the same,
         # for a massive speed boost.
         for offset in range(0, binlength, BinaryDiff.CHUNK_SIZE):
-            if bin1[offset:(offset + BinaryDiff.CHUNK_SIZE)] != bin2[offset:(offset + BinaryDiff.CHUNK_SIZE)]:
-                for i in range(BinaryDiff.CHUNK_SIZE):
+            length = min(BinaryDiff.CHUNK_SIZE, binlength - offset)
+            if bin1[offset:(offset + length)] != bin2[offset:(offset + length)]:
+                for i in range(length):
                     byte1 = bin1[offset + i]
                     byte2 = bin2[offset + i]
 
@@ -91,7 +94,10 @@ class BinaryDiff:
                 # This is a comment, ignore it, unless its a file-size comment
                 patch = patch[1:].strip().lower()
                 if patch.startswith('file size:'):
-                    return int(patch[10:].strip())
+                    try:
+                        return int(patch[10:].strip())
+                    except ValueError:
+                        return None
         return None
 
     @staticmethod
@@ -160,6 +166,7 @@ class BinaryDiff:
         # Finally, return it
         return differences
 
+    @overload
     @staticmethod
     def patch(
         binary: bytes,
@@ -168,6 +175,31 @@ class BinaryDiff:
         reverse: bool = False,
         ignore_size_differences: bool = False,
     ) -> bytes:
+        ...
+
+    @overload
+    @staticmethod
+    def patch(
+        binary: FileBytes,
+        patchlines: List[str],
+        *,
+        reverse: bool = False,
+        ignore_size_differences: bool = False,
+    ) -> FileBytes:
+        ...
+
+    @staticmethod
+    def patch(
+        binary: Union[bytes, FileBytes],
+        patchlines: List[str],
+        *,
+        reverse: bool = False,
+        ignore_size_differences: bool = False,
+    ) -> Union[bytes, FileBytes]:
+        # If we were given filebytes, get a clone of it so we don't modify the input.
+        if isinstance(binary, FileBytes):
+            binary = binary.clone()
+
         # First, grab the differences
         if not ignore_size_differences:
             file_size = BinaryDiff.size(patchlines)
@@ -199,18 +231,31 @@ class BinaryDiff:
                     f"but found {BinaryDiff._hex(binary[offset])}!"
                 )
 
-            if last_patch_end < offset:
-                chunks.append(binary[last_patch_end:offset])
-            chunks.append(new)
-            last_patch_end = offset + 1
+            if isinstance(binary, bytes):
+                if last_patch_end < offset:
+                    chunks.append(binary[last_patch_end:offset])
+                chunks.append(new)
+                last_patch_end = offset + 1
+            elif isinstance(binary, FileBytes):
+                binary[offset:(offset + len(new))] = new
+            else:
+                # This should never happen?
+                raise NotImplementedError("Not implemented!")
 
-        # Return the new data!
-        chunks.append(binary[last_patch_end:])
-        return b"".join(chunks)
+        if isinstance(binary, bytes):
+            # Return the new data!
+            chunks.append(binary[last_patch_end:])
+            return b"".join(chunks)
+        elif isinstance(binary, FileBytes):
+            # We modified the filebytes object in place.
+            return binary
+        else:
+            # This should never happen?
+            raise NotImplementedError("Not implemented!")
 
     @staticmethod
     def can_patch(
-        binary: bytes,
+        binary: Union[bytes, FileBytes],
         patchlines: List[str],
         *,
         reverse: bool = False,
@@ -225,7 +270,11 @@ class BinaryDiff:
                     f"Patch is for binary of size {file_size} but binary is {len(binary)} "
                     f"bytes long!"
                 )
-        differences: List[Tuple[int, Optional[bytes], bytes]] = BinaryDiff._gather_differences(patchlines, reverse)
+
+        try:
+            differences: List[Tuple[int, Optional[bytes], bytes]] = BinaryDiff._gather_differences(patchlines, reverse)
+        except BinaryDiffException as e:
+            return (False, str(e))
 
         # Now, verify the changes to the binary data
         for diff in differences:
@@ -295,7 +344,22 @@ class ByteUtil:
         ]
         return b''.join(chunks)
 
+    @staticmethod
+    def split16bithalves(data: bytes) -> Tuple[bytes, bytes]:
+        length = len(data)
+        return(
+            b''.join(data[x:(x + 2)] for x in range(0, length, 4)),
+            b''.join(data[(x + 2):(x + 4)] for x in range(0, length, 4)),
+        )
+
     @staticmethod
     def combine8bithalves(upper: bytes, lower: bytes) -> bytes:
         chunks = [bytes([upper[i], lower[i]]) for i in range(len(upper))]
         return b''.join(chunks)
+
+    @staticmethod
+    def split8bithalves(data: bytes) -> Tuple[bytes, bytes]:
+        return (
+            bytes([d for d in data[::2]]),
+            bytes([d for d in data[1::2]]),
+        )
